import configparser
from dataclasses import dataclass
from typing import Optional
from pathlib import Path

def read_config(file_path: Path) -> configparser.ConfigParser:
    """读取INI配置文件"""
    config = configparser.ConfigParser()
    config.read(file_path, encoding='utf-8')
    return config

def config_get(config: configparser.ConfigParser, section, key):
    """从配置文件中获取值，支持默认值"""
    return config.get(section, key, fallback=config.get(config.default_section, key, fallback=''))

@dataclass
class NginxConfig:
    name: str = "unknown"
    host: str = "localhost"
    port: int = 80
    output_path: Optional[str] = None
    tcp_nopush: str = "off"
    sendfile: str = "off"
    client_max_body_size: str = "100m"
    proxy_connect_timeout: str = "60s"
    proxy_send_timeout: str = "60s"
    proxy_read_timeout: str = "60s"
    web_socket_proxy: bool = False

def nginx_config(file_path: Path) -> tuple[list[NginxConfig], dict[str, str]]:
    """读取INI配置文件并解析为NginxConfig列表"""
    config = read_config(file_path)
    configs = []

    g_section = config.default_section
    def _config_get(name):
        return config_get(config, g_section, name)

    for section in config.sections():
        g_section = section
        configs.append(NginxConfig(
            name = section,
            host = _config_get('host'),
            port = _config_get('port'),
            web_socket_proxy = _config_get('web_socket_proxy') == 'True',
        ))
    return (configs, {})

@dataclass
class DockerComposeConfig:
    name:           str
    container_name: str
    image:          str
    ports:          Optional[list[str]] = None
    volumes:        Optional[list[str]] = None
    environment:    Optional[list[str]] = None
    extra_hosts:    Optional[list[str]] = None

    export:         Optional[str] = None
    network:        Optional[str] = None

def docker_config(file_path: Path) -> tuple[list[DockerComposeConfig], dict[str, str]]:
    """读取INI配置文件并解析为DockerComposeConfig列表"""
    config = read_config(file_path)
    configs = []

    g_section = ''
    def _config_get(name):
        return config_get(config, g_section, name)

    def _get_item(name) -> list[str]:
        return [i.strip() for i in _config_get(name).split(',') if i.strip()]
    
    network = set()
    for section in config.sections():
        g_section = section
        configs.append(DockerComposeConfig(
            name = section,
            container_name = _config_get('container_name'),
            image =         _config_get('image'),
            volumes =       _get_item('volumes'),
            ports =         _get_item('ports'),
            environment =   _get_item('environment'),
            extra_hosts =    _get_item('extra_hosts'),
            export =        _config_get('export'),
            network =       _config_get('network'),
        ))
        network.add(_config_get('network'))
    return (configs, {'networks': list(network)})

def docker2nginx(file_path: Path) -> tuple[list[NginxConfig], dict[str, str]]:
    "将 docker-compose 服务转换为 nginx 代理"
    """读取INI配置文件并解析为DockerComposeConfig列表"""
    config = read_config(file_path)
    configs = []

    g_section = ''
    def _config_get(name):
        return config_get(config, g_section, name)

    def _get_item(name) -> list[str]:
        return [i.strip() for i in _config_get(name).split(',') if i.strip()]
    
    network = set()
    for section in config.sections():
        if (section == 'nginx'):
            continue
        g_section = section
        configs.append(NginxConfig(
            name = config.get(section, 'nginx_name', fallback=section),
            host = _config_get('container_name'),
            port = _config_get('export'),
        ))
        network.add(_config_get('network'))
    return (configs, {'networks': list(network)})
